@@ -4,18 +4,17 @@ from enum import Enum
 from pathlib import Path
 from re import search
 from sys import stderr
-from time import sleep
 from typing import IO, Union
 from warnings import warn
 
-import psycopg2.errors
 from click import echo, secho
-from psycopg2.extensions import set_wait_callback
-from psycopg2.extras import wait_select
-from psycopg2.sql import SQL, Composable, Composed
+from psycopg.errors import QueryCanceled
+from psycopg.sql import SQL, Composable, Composed
 from rich.console import Console
-from sqlalchemy import MetaData, create_engine, text
+from sqlalchemy import MetaData, text
+from sqlalchemy import create_engine as base_create_engine
 from sqlalchemy.engine import Connection, Engine
+from sqlalchemy.engine.url import make_url
 from sqlalchemy.exc import (
     IntegrityError,
     InternalError,
@@ -29,6 +28,7 @@ from sqlalchemy.sql.elements import ClauseElement, TextClause
 from sqlalchemy_utils import create_database as _create_database
 from sqlalchemy_utils import database_exists, drop_database
 from sqlparse import format, split
+from time import sleep
 
 from macrostrat.utils import cmd, get_logger
 
@@ -196,6 +196,8 @@ def _get_cursor(connectable):
     while hasattr(conn, "driver_connection") or hasattr(conn, "connection"):
         if hasattr(conn, "driver_connection"):
             conn = conn.driver_connection
+        elif conn.connection == conn:
+            break
         else:
             conn = conn.connection
         if callable(conn):
@@ -296,7 +298,7 @@ def _run_sql(connectable, sql, params=None, **kwargs):
         if pre_bind_params is not None:
             if not isinstance(query, SQL):
                 query = SQL(query)
-            # Pre-bind the parameters using PsycoPG2
+            # Pre-bind the parameters using psycopg
             query = query.format(**pre_bind_params)
 
         if isinstance(query, (SQL, Composed)):
@@ -328,9 +330,6 @@ def _run_sql(connectable, sql, params=None, **kwargs):
             )
             continue
 
-        # This only does something for postgresql, but it's harmless to run it for other engines
-        set_wait_callback(wait_select)
-
         try:
             trans = connectable.begin()
         except InvalidRequestError:
@@ -360,7 +359,7 @@ def _run_sql(connectable, sql, params=None, **kwargs):
 
             _print_error(sql_text, err, file=output_file)
         finally:
-            set_wait_callback(None)
+            pass
 
 
 def _should_raise_query_error(err):
@@ -379,7 +378,7 @@ def _should_raise_query_error(err):
     # database backends.
     # Ideally we could handle operational errors more gracefully
     if (
-        isinstance(orig_err, psycopg2.errors.QueryCanceled)
+        isinstance(orig_err, QueryCanceled)
         or getattr(orig_err, "pgcode", None) == "57014"
     ):
         return True
@@ -463,7 +462,14 @@ def run_fixtures(connectable, fixtures: Union[Path, list[Path]], params=None, **
     for fixture in files:
         fn = fixture.relative_to(prefix)
         console.print(f"[cyan bold]{fn}[/]")
-        run_sql_file(connectable, fixture, params, **kwargs)
+        run_sql_file(
+            connectable,
+            fixture,
+            params,
+            output_mode=output_mode,
+            output_file=output_file,
+            **kwargs,
+        )
         console.print()
 
 
@@ -595,6 +601,26 @@ def create_database(url, **kwargs):
     _create_database(url, **kwargs)
 
 
+def create_engine(db_conn, **kwargs):
+    if isinstance(db_conn, Engine):
+        log.info(f"Set up database connection with engine {db_conn.url}")
+        if db_conn.driver == "psycopg2":
+            log.warning(
+                "The psycopg2 driver is deprecated. Please use psycopg3 instead."
+            )
+        return db_conn
+    else:
+        log.info(f"Setting up database connection with URL '{db_conn}'")
+        url = db_conn
+        if isinstance(url, str):
+            url = make_url(url)
+        # Set the driver to psycopg if not already set
+        if url.drivername != "postgresql+psycopg":
+            url = url.set(drivername="postgresql+psycopg")
+
+        return base_create_engine(url, **kwargs)
+
+
 def connection_args(engine):
     """Get PostgreSQL connection arguments for an engine"""
     _psql_flags = {"-U": "username", "-h": "host", "-p": "port", "-P": "password"}
@@ -610,15 +636,24 @@ def connection_args(engine):
     return flags, engine.url.database
 
 
-def db_isready(engine_or_url):
-    args, _ = connection_args(engine_or_url)
-    c = cmd("pg_isready", args, capture_output=True)
-    return c.returncode == 0
+def db_isready(engine_or_url, use_shell_command=False):
+    if use_shell_command:
+        args, _ = connection_args(engine_or_url)
+        c = cmd("pg_isready", args, capture_output=True)
+        return c.returncode == 0
+    # Use a more typical sqlalchemy connection approach
+    engine = create_engine(engine_or_url)
+    try:
+        with engine.connect() as conn:
+            conn.execute(text("SELECT 1"))
+        return True
+    except OperationalError:
+        return False
 
 
-def wait_for_database(engine_or_url, quiet=False):
+def wait_for_database(engine_or_url, *, quiet=False, use_shell_command=False):
     msg = "Waiting for database..."
-    while not db_isready(engine_or_url):
+    while not db_isready(engine_or_url, use_shell_command=use_shell_command):
         if not quiet:
             echo(msg, err=True)
         log.info(msg)
